@@ -11,9 +11,10 @@
 from heapq import nlargest
 from operator import itemgetter
 from functools import wraps
-from itertools import count, chain, izip, repeat, ifilter
+from itertools import count, izip, repeat, ifilter
 
-from brownie.itools import izip_longest, starmap, unique
+from brownie.itools import izip_longest, starmap, unique, chain
+from brownie.abstract import AbstractClassMeta
 
 
 class Missing(object):
@@ -74,15 +75,88 @@ class ImmutableDictMixin(object):
     __setitem__ = __delitem__ = setdefault = update = pop = popitem = clear = \
         raise_immutable
 
-
-class ImmutableDict(ImmutableDictMixin, dict):
-    """An immutable :class:`dict`."""
-
     def __repr__(self):
         content = dict.__repr__(self) if self else ''
         return '%s(%s)' % (self.__class__.__name__, content)
 
 
+class ImmutableDict(ImmutableDictMixin, dict):
+    """An immutable :class:`dict`."""
+    __metaclass__ = AbstractClassMeta
+
+
+class CombinedDictMixin(object):
+    @classmethod
+    def fromkeys(cls, keys, value=None):
+        raise TypeError('cannot create %r instances with .fromkeys()' %
+            cls.__class__.__name__
+        )
+
+    def __init__(self, dicts=None):
+        #: The list of combined dictionaries.
+        self.dicts = [] if dicts is None else list(dicts)
+
+    def __getitem__(self, key):
+        for d in self.dicts:
+            if key in d:
+                return d[key]
+        raise KeyError(key)
+
+    def get(self, key, default=None):
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    def __iter__(self):
+        return unique(chain.from_iterable(d.iterkeys() for d in self.dicts))
+
+    iterkeys = __iter__
+
+    def itervalues(self):
+        for key in self:
+            yield self[key]
+
+    def iteritems(self):
+        for key in self:
+            yield key, self[key]
+
+    def keys(self):
+        return list(self.iterkeys())
+
+    def values(self):
+        return list(self.itervalues())
+
+    def items(self):
+        return list(self.iteritems())
+
+    def __len__(self):
+        return len(self.keys())
+
+    def __contains__(self, key):
+        return any(key in d for d in self.dicts)
+
+    has_key = __contains__
+
+    def __repr__(self):
+        content = repr(self.dicts) if self.dicts else ''
+        return '%s(%s)' % (self.__class__.__name__, content)
+
+
+class CombinedDict(CombinedDictMixin, ImmutableDictMixin, dict):
+    """
+    An immutable :class:`dict` which combines the given `dicts` into one.
+
+    You can use this class to combine dicts of any type, however different
+    interfaces as provided by e.g. :class:`MultiDict` or :class:`Counter` are
+    not supported, the same goes for additional keyword arguments.
+
+    .. versionadded:: 0.2
+    """
+    __metaclass__ = AbstractClassMeta
+    virtual_superclasses = (ImmutableDict, )
+
+
 class MultiDictMixin(object):
     def __init__(self, *args, **kwargs):
         if len(args) > 1:
@@ -293,6 +367,8 @@ class MultiDict(MultiDictMixin, dict):
     a key, you have to use the :class:`list` methods, specific to a
     :class:`MultiDict`.
     """
+    __metaclass__ = AbstractClassMeta
+
     def __repr__(self):
         content = dict.__repr__(self) if self else ''
         return '%s(%s)' % (self.__class__.__name__, content)
@@ -317,9 +393,44 @@ class ImmutableMultiDictMixin(ImmutableDictMixin, MultiDictMixin):
 
 class ImmutableMultiDict(ImmutableMultiDictMixin, dict):
     """An immutable :class:`MultiDict`."""
-    def __repr__(self):
-        content = dict.__repr__(self) if self else ''
-        return '%s(%s)' % (self.__class__.__name__, content)
+    __metaclass__ = AbstractClassMeta
+
+    virtual_superclasses = (MultiDict, ImmutableDict)
+
+
+class CombinedMultiDict(CombinedDictMixin, ImmutableMultiDictMixin, dict):
+    """
+    An :class:`ImmutableMultiDict` which combines the given `dicts` into one.
+
+    .. versionadded:: 0.2
+    """
+    __metaclass__ = AbstractClassMeta
+
+    virtual_superclasses = (ImmutableMultiDict, )
+
+    def getlist(self, key):
+        return sum((d.getlist(key) for d in self.dicts), [])
+
+    def iterlists(self):
+        result = OrderedDict()
+        for d in self.dicts:
+            for key, values in d.iterlists():
+                result.setdefault(key, []).extend(values)
+        return result.iteritems()
+
+    def iterlistvalues(self):
+        for key in self:
+            yield self.getlist(key)
+
+    def iteritems(self, multi=False):
+        for key in self:
+            if multi:
+                yield key, self.getlist(key)
+            else:
+                yield key, self[key]
+
+    def items(self, multi=False):
+        return list(self.iteritems(multi))
 
 
 class _Link(object):
@@ -505,12 +616,35 @@ class OrderedDict(dict):
         return '%s(%s)' % (self.__class__.__name__, content)
 
 
+class ImmutableOrderedDict(ImmutableDictMixin, OrderedDict):
+    """
+    An immutable :class:`OrderedDict`.
+
+    .. versionadded:: 0.2
+    """
+    __metaclass__ = AbstractClassMeta
+
+    virtual_superclasses = (ImmutableDict, )
+
+    __repr__ = OrderedDict.__repr__
+
+
 class OrderedMultiDict(MultiDictMixin, OrderedDict):
     """An ordered :class:`MultiDict`."""
+    __metaclass__ = AbstractClassMeta
+
+    virtual_superclasses = (MultiDict, )
 
 
 class ImmutableOrderedMultiDict(ImmutableMultiDictMixin, OrderedDict):
     """An immutable :class:`OrderedMultiDict`."""
+    __metaclass__ = AbstractClassMeta
+
+    virtual_superclasses = (ImmutableMultiDict, OrderedMultiDict)
+
+    def __repr__(self):
+        content = repr(self.items()) if self else ''
+        return '%s(%s)' % (self.__class__.__name__, content)
 
 
 class Counter(dict):
@@ -999,3 +1133,164 @@ class LazyList(object):
         )
 
     del exhausting
+
+
+class OrderedSet(object):
+    """
+    A :class:`set` which remembers insertion order.
+
+    .. versionadded:: 0.2
+    """
+    def requires_set(func):
+        @wraps(func)
+        def wrapper(self, other):
+            if isinstance(other, (self.__class__, set, frozenset)):
+                return func(self, other)
+            return NotImplemented
+        return wrapper
+
+    def __init__(self, iterable=None):
+        self._orderedmap = OrderedDict.fromkeys(iterable or ())
+
+    def __len__(self):
+        return len(self._orderedmap)
+
+    def __contains__(self, element):
+        return element in self._orderedmap
+
+    def add(self, element):
+        self._orderedmap[element] = None
+
+    def remove(self, element):
+        del self._orderedmap[element]
+
+    def discard(self, element):
+        self._orderedmap.pop(element, None)
+
+    def pop(self, last=True):
+        """
+        Returns the last element if `last` is ``True``, the first otherwise.
+        """
+        if not self:
+            raise KeyError('set is empty')
+        element = self._orderedmap.popitem(last=last)[0]
+        return element
+
+    def clear(self):
+        self._orderedmap.clear()
+
+    def update(self, *others):
+        for other in others:
+            for element in other:
+                self._orderedmap[element] = None
+
+    def copy(self):
+        return self.__class__(self)
+
+    @requires_set
+    def __ior__(self, other):
+        self.update(other)
+        return self
+
+    def issubset(self, other):
+        return all(element in other for element in self)
+
+    @requires_set
+    def __le__(self, other):
+        return self.issubset(other)
+
+    @requires_set
+    def __lt__(self, other):
+        return self.issubset(other) and self != other
+
+    def issuperset(self, other):
+        return all(element in self for element in other)
+
+    @requires_set
+    def __ge__(self, other):
+        return self.issuperset(other)
+
+    @requires_set
+    def __gt__(self, other):
+        return self.issuperset(other) and self != other
+
+    def union(self, *others):
+        return self.__class__(chain.from_iterable((self, ) + others))
+
+    @requires_set
+    def __or__(self, other):
+        return self.union(other)
+
+    def intersection(self, *others):
+        def intersect(a, b):
+            result = self.__class__()
+            smallest = min([a, b], key=len)
+            for element in max([a, b], key=len):
+                if element in smallest:
+                    result.add(element)
+            return result
+        return reduce(intersect, others, self)
+
+    @requires_set
+    def __and__(self, other):
+        return self.intersection(other)
+
+    @requires_set
+    def __iand__(self, other):
+        intersection = self.intersection(other)
+        self.clear()
+        self.update(intersection)
+        return self
+
+    def difference(self, *others):
+        return self.__class__(
+            key for key in self if not any(key in s for s in others)
+        )
+
+    @requires_set
+    def __sub__(self, other):
+        return self.difference(other)
+
+    @requires_set
+    def __isub__(self, other):
+        diff = self.difference(other)
+        self.clear()
+        self.update(diff)
+        return self
+
+    def symmetric_difference(self, other):
+        other = self.__class__(other)
+        return self.__class__(chain(self - other, other - self))
+
+    @requires_set
+    def __xor__(self, other):
+        return self.symmetric_difference(other)
+
+    @requires_set
+    def __ixor__(self, other):
+        diff = self.symmetric_difference(other)
+        self.clear()
+        self.update(diff)
+        return self
+
+    def __iter__(self):
+        return iter(self._orderedmap)
+
+    def __reversed__(self):
+        return reversed(self._orderedmap)
+
+    def __eq__(self, other):
+        if isinstance(other, self.__class__):
+            return len(self) == len(other) and list(self) == list(other)
+        return set(self) == other
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    __hash__ = None
+
+    def __repr__(self):
+        content = repr(list(self)) if self else ''
+        return '%s(%s)' % (self.__class__.__name__, content)
+
+    del requires_set
